@@ -32,6 +32,7 @@
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-hsv-utils.h"
 #include "e-table-item.h"
+#include "gal/a11y/e-table/gal-a11y-e-cell-toggle.h"
 
 #define PARENT_TYPE e_cell_get_type ()
 
@@ -79,7 +80,6 @@ etog_new_view (ECell *ecell, ETableModel *table_model, void *e_table_item_view)
 	toggle_view->cell_view.e_table_model = table_model;
 	toggle_view->cell_view.e_table_item_view = e_table_item_view;
 	toggle_view->canvas = canvas;
-	
 	toggle_view->pixmap_cache = g_new (GdkPixmap *, etog->n_states * CACHE_SEQ_COUNT);
 	for (i = 0; i < etog->n_states * CACHE_SEQ_COUNT; i++)
 		toggle_view->pixmap_cache[i] = NULL;
@@ -136,7 +136,7 @@ check_cache (ECellToggleView *toggle_view, int image_seq, int cache_seq)
 	if (PIXMAP_CACHE (toggle_view, cache_seq, image_seq) == NULL) {
 		GdkPixbuf *image = etog->images[image_seq];
 		GdkPixbuf *flat;
-		guint32 color = 0xffffff;
+		GdkColor  color;
 		int width = gdk_pixbuf_get_width (image);
 		int height = gdk_pixbuf_get_height (image);
 
@@ -147,35 +147,18 @@ check_cache (ECellToggleView *toggle_view, int image_seq, int cache_seq)
 		
 		switch (cache_seq % 3) {
 		case 0:
-			color = RGB_COLOR (GTK_WIDGET (toggle_view->canvas)->style->bg [GTK_STATE_SELECTED]);
+			color = GTK_WIDGET (toggle_view->canvas)->style->bg [GTK_STATE_SELECTED];
 			break;
 		case 1:
-			color = RGB_COLOR (GTK_WIDGET (toggle_view->canvas)->style->bg [GTK_STATE_ACTIVE]);
+			color = GTK_WIDGET (toggle_view->canvas)->style->bg [GTK_STATE_ACTIVE];
 			break;
 		case 2:
-			color = RGB_COLOR (GTK_WIDGET (toggle_view->canvas)->style->base [GTK_STATE_NORMAL]);
+			color = GTK_WIDGET (toggle_view->canvas)->style->base [GTK_STATE_NORMAL];
 			break;
 		}
 
 		if (cache_seq >= 3) {
-			double r, g, b, h, s, v;
-			r = ((color >> 16) & 0xff) / 255.0f;
-			g = ((color >> 8) & 0xff) / 255.0f;
-			b = (color & 0xff) / 255.0f;
-
-			e_rgb_to_hsv (r, g, b, &h, &s, &v);
-	
-			if (v - 0.05f < 0) {
-				v += 0.05f;
-			} else {
-				v -= 0.05f;
-			}
-
-			e_hsv_to_rgb (h, s, v, &r, &g, &b);
-
-			color = ((((int)(r * 255.0f)) & 0xff) << 16) +
-				((((int)(g * 255.0f)) & 0xff) << 8) +
-				(((int)(b * 255.0f)) & 0xff);
+			e_hsv_tweak (&color, 0.0f, 0.0f, -0.07f);
 		}
 
 		flat = gdk_pixbuf_composite_color_simple (image,
@@ -183,7 +166,7 @@ check_cache (ECellToggleView *toggle_view, int image_seq, int cache_seq)
 							  GDK_INTERP_BILINEAR,
 							  255,
 							  1,
-							  color, color);
+							  RGB_COLOR (color), RGB_COLOR (color));
 
 		gdk_pixbuf_render_to_drawable (flat, PIXMAP_CACHE (toggle_view, cache_seq, image_seq),
 					       toggle_view->gc,
@@ -380,6 +363,21 @@ etog_max_width (ECellView *ecell_view, int model_col, int view_col)
 	return max_width;
 }
 
+static void
+etog_style_set (ECellView *ecell_view, GtkStyle *previous_style)
+{
+	ECellToggle *toggle = E_CELL_TOGGLE (ecell_view->ecell);
+	ECellToggleView *toggle_view = (ECellToggleView *) ecell_view;
+	int i;
+
+	for (i = 0; i < toggle->n_states * CACHE_SEQ_COUNT; i++) {
+		if (toggle_view->pixmap_cache[i]) {
+			gdk_pixmap_unref (toggle_view->pixmap_cache[i]);
+			toggle_view->pixmap_cache[i] = NULL;
+		}
+	}
+}
+
 static void
 etog_finalize (GObject *object)
 {
@@ -397,21 +395,6 @@ etog_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
-etog_style_set (ECellView *ecell_view, GtkStyle *previous_style)
-{
-	ECellToggle *toggle = E_CELL_TOGGLE (ecell_view->ecell);
-	ECellToggleView *toggle_view = (ECellToggleView *) ecell_view;
-	int i;
-
-	for (i = 0; i < toggle->n_states * CACHE_SEQ_COUNT; i++) {
-		if (toggle_view->pixmap_cache[i]) {
-			gdk_pixmap_unref (toggle_view->pixmap_cache[i]);
-			toggle_view->pixmap_cache[i] = NULL;
-		}
-	}
-}
-
 static void
 e_cell_toggle_class_init (GtkObjectClass *object_class)
 {
@@ -431,7 +414,10 @@ e_cell_toggle_class_init (GtkObjectClass *object_class)
 	ecc->max_width  = etog_max_width;
 	ecc->style_set  = etog_style_set;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_ref (PARENT_TYPE);
+	gal_a11y_e_cell_registry_add_cell_type (NULL,
+                                                E_CELL_TOGGLE_TYPE,
+                                                gal_a11y_e_cell_toggle_new);
 }
 
 static void
@@ -494,7 +480,7 @@ e_cell_toggle_construct (ECellToggle *etog, int border, int n_states, GdkPixbuf
 ECell *
 e_cell_toggle_new (int border, int n_states, GdkPixbuf **images)
 {
-	ECellToggle *etog = gtk_type_new (e_cell_toggle_get_type ());
+	ECellToggle *etog = g_object_new (E_CELL_TOGGLE_TYPE, NULL);
 
 	e_cell_toggle_construct (etog, border, n_states, images);
 
