@@ -60,6 +60,9 @@
 #define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)))
 #endif
 
+static void eti_check_cursor_bounds (ETableItem *eti);
+static void eti_cancel_drag_due_to_model_change (ETableItem *eti);
+
 /* FIXME: Do an analysis of which cell functions are needed before
    realize and make sure that all of them are doable by all the cells
    and that all of the others are only done after realization. */
@@ -353,7 +356,15 @@ eti_attach_cell_views (ETableItem *eti)
 
 	g_assert (eti->header);
 	g_assert (eti->table_model);
-	
+
+	/* this is just c&p from model pre change, but it fixes things */
+	eti_cancel_drag_due_to_model_change (eti);
+	eti_check_cursor_bounds (eti);
+	if (eti_editing (eti))
+		e_table_item_leave_edit_(eti);
+	eti->motion_row = -1;
+	eti->motion_col = -1;
+
 	/*
 	 * Now realize the various ECells
 	 */
@@ -1068,6 +1079,8 @@ eti_table_model_pre_change (ETableModel *table_model, ETableItem *eti)
 	eti_check_cursor_bounds (eti);
 	if (eti_editing (eti))
 		e_table_item_leave_edit_(eti);
+	eti->motion_row = -1;
+	eti->motion_col = -1;
 	eti_freeze (eti);
 }
 
@@ -2680,6 +2693,10 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		case GDK_Tab:
 		case GDK_KP_Tab:
 		case GDK_ISO_Left_Tab:
+			if ((e->key.state & GDK_CONTROL_MASK) != 0) {
+				return_val = FALSE;
+				break;
+			}
 			if (eti->cursor_mode == E_CURSOR_SPREADSHEET) {
 				if ((e->key.state & GDK_SHIFT_MASK) != 0){
 				/* shift tab */
