@@ -24,11 +24,12 @@
 #include <config.h>
 #include <string.h>
 #include "e-table-memory-store.h"
-
-#define PARENT_TYPE e_table_memory_get_type ()
+#include "gal/util/e-util.h"
 
 #define STORE_LOCATOR(etms, col, row) (*((etms)->priv->store + (row) * (etms)->priv->col_count + (col)))
 
+static ETableMemoryClass *parent_class;
+
 struct _ETableMemoryStorePrivate {
 	int col_count;
 	ETableMemoryStoreColumnInfo *columns;
@@ -47,7 +48,7 @@ duplicate_value (ETableMemoryStore *etms, int col, const void *val)
 		return (void *) val;
 	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_OBJECT:
 		if (val)
-			gtk_object_ref ((void *) val);
+			g_object_ref ((void *) val);
 		return (void *) val;
 	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_CUSTOM:
 		if (etms->priv->columns[col].custom.duplicate_value)
@@ -59,6 +60,31 @@ duplicate_value (ETableMemoryStore *etms, int col, const void *val)
 	return (void *) val;
 }
 
+static void
+free_value (ETableMemoryStore *etms, int col, void *value)
+{
+	switch (etms->priv->columns[col].type) {
+	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_STRING:
+		g_free (value);
+		break;
+	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_PIXBUF:
+		if (value)
+			gdk_pixbuf_unref (value);
+		break;
+	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_OBJECT:
+		if (value)
+			g_object_unref (value);
+		break;
+	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_CUSTOM:
+		if (etms->priv->columns[col].custom.free_value)
+			etms->priv->columns[col].custom.free_value (E_TABLE_MODEL (etms), col, value, NULL);
+		break;
+	default:
+		break;
+	}
+}
+
+
 static int
 etms_column_count (ETableModel *etm)
 {
@@ -109,25 +135,7 @@ etms_free_value (ETableModel *etm, int col, void *value)
 {
 	ETableMemoryStore *etms = E_TABLE_MEMORY_STORE(etm);
 
-	switch (etms->priv->columns[col].type) {
-	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_STRING:
-		g_free (value);
-		break;
-	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_PIXBUF:
-		if (value)
-			gdk_pixbuf_unref (value);
-		break;
-	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_OBJECT:
-		if (value)
-			gtk_object_unref (value);
-		break;
-	case E_TABLE_MEMORY_STORE_COLUMN_TYPE_CUSTOM:
-		if (etms->priv->columns[col].custom.free_value)
-			etms->priv->columns[col].custom.free_value (E_TABLE_MODEL (etms), col, value, NULL);
-		break;
-	default:
-		break;
-	}
+	free_value (etms, col, value);
 }
 
 static void *
@@ -212,6 +220,23 @@ etms_append_row (ETableModel *etm, ETableModel *source, int row)
 	e_table_memory_store_insert_array (etms, row_count, new_data, NULL);
 }
 
+static void
+etms_finalize (GObject *obj)
+{
+	ETableMemoryStore *etms = (ETableMemoryStore *) obj;
+
+	if (etms->priv) {
+		e_table_memory_store_clear (etms);
+
+		g_free (etms->priv->columns);
+		g_free (etms->priv->store);
+		g_free (etms->priv);
+	}
+
+	if (G_OBJECT_CLASS (parent_class)->finalize)
+		G_OBJECT_CLASS (parent_class)->finalize (obj);
+}
+	
 static void
 e_table_memory_store_init (ETableMemoryStore *etms)
 {
@@ -223,10 +248,14 @@ e_table_memory_store_init (ETableMemoryStore *etms)
 }
 
 static void
-e_table_memory_store_class_init (GtkObjectClass *object_class)
+e_table_memory_store_class_init (GObjectClass *object_class)
 {
 	ETableModelClass *model_class = (ETableModelClass *) object_class;
 
+	parent_class = g_type_class_peek_parent (object_class);
+
+	object_class->finalize	      = etms_finalize;
+
 	model_class->column_count     = etms_column_count;
 	model_class->value_at         = etms_value_at;
 	model_class->set_value_at     = etms_set_value_at;
@@ -239,28 +268,7 @@ e_table_memory_store_class_init (GtkObjectClass *object_class)
 	model_class->append_row       = etms_append_row;
 }
 
-GtkType
-e_table_memory_store_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type){
-		GtkTypeInfo info = {
-			"ETableMemoryStore",
-			sizeof (ETableMemoryStore),
-			sizeof (ETableMemoryStoreClass),
-			(GtkClassInitFunc) e_table_memory_store_class_init,
-			(GtkObjectInitFunc) e_table_memory_store_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (PARENT_TYPE, &info);
-	}
-
-	return type;
-}
+E_MAKE_TYPE(e_table_memory_store, "ETableMemoryStore", ETableMemoryStore, e_table_memory_store_class_init, e_table_memory_store_init, E_TABLE_MEMORY_TYPE)
 
 /**
  * e_table_memory_store_new:
@@ -291,14 +299,12 @@ e_table_memory_store_get_type (void)
 ETableModel *
 e_table_memory_store_new (ETableMemoryStoreColumnInfo *columns)
 {
-	ETableMemoryStore *et;
-
-	et = gtk_type_new (e_table_memory_store_get_type ());
+	ETableMemoryStore *et = g_object_new (E_TABLE_MEMORY_STORE_TYPE, NULL);
 
 	if (e_table_memory_store_construct (et, columns)) {
 		return (ETableModel *) et;
 	} else {
-		gtk_object_unref (GTK_OBJECT (et));
+		g_object_unref (et);
 		return NULL;
 	}
 }
@@ -412,10 +418,137 @@ e_table_memory_store_insert_adopt (ETableMemoryStore *etms, int row, gpointer da
 	g_free (store);
 }
 
+/**
+ * e_table_memory_store_change_array:
+ * @etms: the ETabelMemoryStore.
+ * @row:  the row we're changing.
+ * @store: an array of new values to fill the row
+ * @data: the new closure to associate with this row.
+ *
+ * frees existing values associated with a row and replaces them with
+ * duplicates of the values in store.
+ *
+ */
+void
+e_table_memory_store_change_array (ETableMemoryStore *etms, int row, void **store, gpointer data)
+{
+	int i;
+
+	g_return_if_fail (row >= 0 && row < e_table_model_row_count (E_TABLE_MODEL (etms)));
+
+	e_table_model_pre_change (E_TABLE_MODEL (etms));
+
+	for (i = 0; i < etms->priv->col_count; i++) {
+		free_value (etms, i, STORE_LOCATOR(etms, i, row));
+		STORE_LOCATOR(etms, i, row) = duplicate_value(etms, i, store[i]);
+	}
+
+	e_table_memory_set_data (E_TABLE_MEMORY (etms), row, data);
+	e_table_model_row_changed (E_TABLE_MODEL (etms), row);
+}
+
+/**
+ * e_table_memory_store_change:
+ * @etms: the ETabelMemoryStore.
+ * @row:  the row we're changing.
+ * @data: the new closure to associate with this row.
+ *
+ * a varargs version of e_table_memory_store_change_array.  you must
+ * pass in etms->col_count args.
+ */
+void
+e_table_memory_store_change (ETableMemoryStore *etms, int row, gpointer data, ...)
+{
+	void **store;
+	va_list args;
+	int i;
+
+	g_return_if_fail (row >= 0 && row < e_table_model_row_count (E_TABLE_MODEL (etms)));
+
+	store = g_new0 (void *, etms->priv->col_count + 1);
+
+	va_start (args, data);
+	for (i = 0; i < etms->priv->col_count; i++) {
+		store[i] = va_arg (args, void *);
+	}
+	va_end (args);
+
+	e_table_memory_store_change_array (etms, row, store, data);
+
+	g_free (store);
+}
+
+/**
+ * e_table_memory_store_change_adopt_array:
+ * @etms: the ETableMemoryStore
+ * @row: the row we're changing.
+ * @store: an array of new values to fill the row
+ * @data: the new closure to associate with this row.
+ *
+ * frees existing values for the row and stores the values from store
+ * into it.  This function differs from
+ * e_table_memory_storage_change_adopt_array in that it does not
+ * duplicate the data.
+ */
+void
+e_table_memory_store_change_adopt_array (ETableMemoryStore *etms, int row, void **store, gpointer data)
+{
+	int i;
+
+	g_return_if_fail (row >= 0 && row < e_table_model_row_count (E_TABLE_MODEL (etms)));
+
+	for (i = 0; i < etms->priv->col_count; i++) {
+		free_value (etms, i, STORE_LOCATOR(etms, i, row));
+		STORE_LOCATOR(etms, i, row) = store[i];
+	}
+
+	e_table_memory_set_data (E_TABLE_MEMORY (etms), row, data);
+	e_table_model_row_changed (E_TABLE_MODEL (etms), row);
+}
+
+/**
+ * e_table_memory_store_change_adopt
+ * @etms: the ETabelMemoryStore.
+ * @row:  the row we're changing.
+ * @data: the new closure to associate with this row.
+ *
+ * a varargs version of e_table_memory_store_change_adopt_array.  you
+ * must pass in etms->col_count args.
+ */
+void
+e_table_memory_store_change_adopt (ETableMemoryStore *etms, int row, gpointer data, ...)
+{
+	void **store;
+	va_list args;
+	int i;
+
+	g_return_if_fail (row >= 0 && row < e_table_model_row_count (E_TABLE_MODEL (etms)));
+
+	store = g_new0 (void *, etms->priv->col_count + 1);
+
+	va_start (args, data);
+	for (i = 0; i < etms->priv->col_count; i++) {
+		store[i] = va_arg (args, void *);
+	}
+	va_end (args);
+
+	e_table_memory_store_change_adopt_array (etms, row, store, data);
+
+	g_free (store);
+}
+
 void
 e_table_memory_store_remove (ETableMemoryStore *etms, int row)
 {
-	int row_count;
+	ETableModel *model;
+	int column_count, row_count;
+	int i;
+
+	model = E_TABLE_MODEL (etms);
+	column_count = e_table_model_column_count (model);
+
+	for (i = 0; i < column_count; i ++)
+		e_table_model_free_value (model, i, e_table_model_value_at (model, i, row));
 
 	row_count = e_table_model_row_count (E_TABLE_MODEL (etms)) - 1;
 	memmove (etms->priv->store + etms->priv->col_count * row,
@@ -429,6 +562,20 @@ e_table_memory_store_remove (ETableMemoryStore *etms, int row)
 void
 e_table_memory_store_clear (ETableMemoryStore *etms)
 {
+	ETableModel *model;
+	int row_count, column_count;
+	int i, j;
+
+	model = E_TABLE_MODEL (etms);
+	row_count = e_table_model_row_count (model);
+	column_count = e_table_model_column_count (model);
+
+	for (i = 0; i < row_count; i ++) {
+		for (j = 0; j < column_count; j ++) {
+			e_table_model_free_value (model, j, e_table_model_value_at (model, j, i));
+		}
+	}
+
 	e_table_memory_clear (E_TABLE_MEMORY (etms));
 
 	g_free (etms->priv->store);
